import { SmsManager } from '@byteowls/capacitor-sms';
import { EntityUpdateType } from '@models/providers/entity-update-event.models';
import { ScrollService } from '@providers/scroll/scroll.service';
import { Component, OnInit } from '@angular/core';
import { LocationTrackingService } from '@providers/location-tracking/location-tracking.service';
import { MenuController, Platform, AlertController, NavController } from '@ionic/angular';
import { AccountTechniciansStoreService } from '@providers/account/account-technicians-store.service';
import { AuthenticationServiceProvider } from '@providers/authentication/authentication.service';
import { Router } from '@angular/router';
import { StorageService } from '@providers/storage/storage.service';
import { asset } from '@scorpion/utils';
import { routes } from '@scorpion/ionic/features/routes';
import { AppStartOfflineRequestsService } from '@scorpion/shared/providers/offline-manager/app-start-requests.service';
import { filter, map, scan, take, pairwise, switchMap } from 'rxjs/operators';
import { EventsService } from '@scorpion/shared/providers/events/events.service';
import { Plugins, registerWebPlugin } from '@capacitor/core';
import { environment } from '@scorpion/shared/environments/environment';

const { Keyboard, SplashScreen } = Plugins;
@Component({
    templateUrl: 'app.component.html',
    styleUrls: ['app.component.scss'],
    selector: 'app-root'
})
export class ScorpionCrmMobilePage implements OnInit {
    rootPage: {};
    technicianName = '';
    technicianInitials = '';
    companyPhoneNumber = '';
    companyEmail = '';
    businessName = '';
    imageUrl = '';
    trackingToggleEnabled: boolean;
    currentUsername = '';
    iconCallCircle = asset('icon/CallCircle.svg');
    iconMessageCircle = asset('icon/MessageCircle.svg');
    iconLockCircle = asset('icon/LockCircle.svg');
    iconXCircle = asset('icon/XCircle.svg');

    constructor(
        private platform: Platform,
        private storage: StorageService,
        private authenticationServiceProvider: AuthenticationServiceProvider,
        private locationTrackingService: LocationTrackingService,
        private eventsService: EventsService,
        private menuCtrl: MenuController,
        private router: Router,
        private accountTechniciansStoreService: AccountTechniciansStoreService,
        private scrollService: ScrollService,
        private alertCtrl: AlertController,
        private navCtrl: NavController,
        public appStart: AppStartOfflineRequestsService // need to inject it here, otherwise it won't be executed (not injected to any other components\services)
    ) {}

    async ngOnInit() {
        registerWebPlugin(SmsManager);
        this.enableRouterLogging();
        await this.authenticationServiceProvider.restoreSession();
        await this.accountTechniciansStoreService.readFromStorage();

        this.platform.ready().then(() => {
            if (this.platform.is('cordova')) {
                this.scrollService.listenForKeyboard();
                Keyboard.setAccessoryBarVisible({ isVisible: true });
                SplashScreen.hide();
            }

            this.listenForTechnicianInfoUpdate();
            this.listenForLocationTrackingSettingsUpdated();
        });

        this.authenticationServiceProvider.isLoggedIn().subscribe(async loggedIn => {
            if (loggedIn) {
                this.trackingToggleEnabled = await this.locationTrackingService.trackingToggleEnabled();
                await this.accountTechniciansStoreService.fetchTechnicians();
            } else {
                if (this.authenticationServiceProvider.visited) {
                    this.authenticationServiceProvider.sessionExpired = false;
                    await this.accountTechniciansStoreService.reset();
                    this.navCtrl.navigateRoot(routes.login());
                } else {
                    this.navCtrl.navigateRoot([routes.landing()]);
                }
            }
        });

        this.authenticationServiceProvider.tokenSubject.subscribe(async (token: string) => {
            const locationTrackingEnabled = await this.locationTrackingService.trackingToggleEnabled();
            if (token && locationTrackingEnabled) {
                setTimeout(() => {
                    // Waiting a bit so the alert doesn't appear underneath the spinner
                    this.locationTrackingService.startTracking();
                }, 300);
            } else {
                this.locationTrackingService.stopTracking();
            }
        });
    }

    listenForTechnicianInfoUpdate() {
        this.eventsService
            .getUpdatesForEntityChanges([EntityUpdateType.TechnicianInfo])
            .pipe(
                switchMap(() =>
                    Promise.all([
                        this.storage.get('technicianName'),
                        this.storage.get('companyPhoneNumber'),
                        this.storage.get('companyEmail'),
                        this.storage.get('businessName'),
                        this.storage.get('profileImageUrl'),
                        this.storage.get('technicianInitials')
                    ])
                )
            )
            .subscribe(([technicianName, companyPhoneNumber, companyEmail, businessName, profileImageUrl, technicianInitials]) => {
                this.technicianName = technicianName;
                this.companyPhoneNumber = companyPhoneNumber;
                this.businessName = businessName;
                this.imageUrl = profileImageUrl;
                this.companyEmail = companyEmail;
                this.technicianInitials = technicianInitials;
            });
    }

    listenForLocationTrackingSettingsUpdated() {
        this.eventsService
            .getUpdatesForEntityChanges([EntityUpdateType.LocationTracking])
            .pipe(switchMap(() => this.locationTrackingService.trackingToggleEnabled()))
            .subscribe(enabled => (this.trackingToggleEnabled = enabled));
    }

    async showNumber() {
        const alert = await this.alertCtrl.create({
            header: `Call ${this.businessName}?`,
            subHeader: `Number is ${this.companyPhoneNumber}`,
            buttons: [
                {
                    text: 'Cancel',
                    role: 'cancel',
                    cssClass: 'secondary'
                },
                {
                    text: 'Call',
                    role: 'confirm'
                }
            ]
        });
        alert.onDidDismiss().then(data => {
            if (data.role === 'confirm') {
                this.callBusiness();
            }
        });
        await alert.present();
    }
    private callBusiness() {
        window.open(`tel:${this.companyPhoneNumber}`, '_system');
    }

    messageBusiness() {
        const options = {
            replaceLineBreaks: false,
            android: {
                intent: 'INTENT'
            }
        };
        SmsManager.send({ numbers: [this.companyPhoneNumber], text: '' });
    }

    logOut() {
        this.menuCtrl.close();
        this.authenticationServiceProvider.logout();
        this.navCtrl.navigateRoot(routes.login());
    }

    async toggleLocationTracking(enableLocationTracking: boolean) {
        await this.locationTrackingService.setTrackingToggleEnabled(enableLocationTracking);
    }

    /**
     * This is for debugging purposes.
     * If some module is configured wrong with routing, router may go into infinite loop.
     * This will allow you to see the error if it occurs
     */
    private enableRouterLogging() {
        if (!environment.production) {
            this.router.events
                .pipe(
                    map(() => new Date()),
                    pairwise(),
                    scan((acc, [oldDate, newDate]) => {
                        if (newDate.getTime() - oldDate.getTime() < 1000) {
                            return acc + 1;
                        }
                        return 1;
                    }, 0),
                    filter(count => count > 300),
                    take(1)
                )
                .subscribe(() => console.warn(`Probably there's a problem with router. Try to set "enableTracing" to true in "app-routing.module.ts"`));
        }
    }
}
